--[[
	MyRolePlay 4 (C) 2010-2018 Etarna Moonshyne <etarna@moonshyne.org>
	Licensed under GNU General Public Licence version 2 or, at your option, any later version

	UI_EditFrames.lua - Supporting EditFrames for the MyRolePlayCharacterFrame
]]

local L = mrp.L

local function emptynil( x ) return x ~= "" and x or nil end

local uipbt = mrp.WoWTOC >= 50001 and "UIPanelButtonTemplate" or "UIPanelButtonTemplate2"

local previousGlanceIcons = {} -- Save previous icons here temporarily until the editor is closed so we can restore them if they cancel.

function mrp:ClearGlance(glance) -- Wipe a glance's data using the clear button.
	_G["Glance" .. glance .. "Title"]:SetText("")
	_G["Glance" .. glance .. "Descrip"].editbox:SetText("")
	_G["Glance" .. glance .. "IconTexture"]:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
	mrpSaved.Profiles[mrpSaved.SelectedProfile]["glances"][glance]["Icon"] = "Interface\\Icons\\INV_Misc_QuestionMark"
end
	

-- Create the EditFrames to sit alongside MyRolePlayCharacterFrame
function mrp:CreateEditFrames()
	-- MyRolePlayGlanceEditFrame
	if not MyRolePlayGlanceEditFrame then
		local mef = CreateFrame( "Frame", "MyRolePlayGlanceEditFrame", MyRolePlayCharacterFrame )
		mef:Hide()
		mef:SetPoint( "TOPLEFT", MyRolePlayCharacterFrame, "TOPLEFT", 332, -20 )
		mef:SetPoint( "BOTTOM", CharacterFrameInset, "BOTTOM", 0, 30 )
		mef:SetPoint( "RIGHT", CharacterFrame, "RIGHT", -8, 0 )
		mef:EnableDrawLayer("ARTWORK")

		mef:SetScript("OnShow", function(self)
			CharacterFrame:SetWidth( 700 )
			CharacterFrame.Expanded = true
			UpdateUIPanelPositions(CharacterFrame)
		end	)
		mef:SetScript("OnHide", function(self)
			CharacterFrame:SetWidth( PANEL_DEFAULT_WIDTH )
			CharacterFrame.Expanded = false
			UpdateUIPanelPositions(CharacterFrame)
		end	)

		mef:EnableDrawLayer("OVERLAY")
		
		----------------------------------
		-- Glance 1
		----------------------------------
		
		mef.Glance1Icon = CreateFrame("Frame", "Glance1Icon", mef)
		mef.Glance1Icon:SetPoint( "TOPLEFT", MyRolePlayGlanceEditFrame, "TOPLEFT", 5, -15 )
		mef.Glance1Icon:SetHeight(64)
		mef.Glance1Icon:SetWidth(64)
		local glance1IconTexture = mef.Glance1Icon:CreateTexture("Glance1IconTexture", "ARTWORK")
		glance1IconTexture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		glance1IconTexture:SetAllPoints(mef.Glance1Icon)
		
		local glance1IconHighlightTexture = mef.Glance1Icon:CreateTexture("Glance1IconHighlightTexture", "OVERLAY")
		glance1IconHighlightTexture:SetTexture("Interface\\AddOns\\MyRolePlay\\Artwork\\GlanceHighlight.blp")
		glance1IconHighlightTexture:SetAllPoints(mef.Glance1Icon)
		glance1IconHighlightTexture:Hide()
		
		mef.Glance1Icon:SetScript("OnMouseDown", function (self)
			mrp_iconselector_show(mrp_IconCallbackGlance1Icon, mrpSaved.Profiles[mrpSaved.SelectedProfile]["glances"][1]["Icon"] or nil)
		end )
		
		mef.Glance1Icon:SetScript("OnEnter", function (self)
			glance1IconHighlightTexture:Show()
		end )
		
		mef.Glance1Icon:SetScript("OnLeave", function (self)
			glance1IconHighlightTexture:Hide()
		end )


		mef.Glance1Title = CreateFrame( "EditBox", "Glance1Title", mef )
		mef.Glance1Title:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance1Title:SetPoint( "TOPLEFT", mef.Glance1Icon, "TOPRIGHT", 3, 0 )
		mef.Glance1Title:SetHeight( 25 )
		mef.Glance1Title:SetWidth( 280 )
		mef.Glance1Title:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance1Title:EnableMouse(true)
		mef.Glance1Title:SetAutoFocus(false)
		mef.Glance1Title:SetMultiLine(false)
		mef.Glance1Title:SetFontObject( "GameFontHighlight" )

		mef.Glance1Title:SetScript( "OnEscapePressed", EditBox_ClearFocus )
		
		mef.Glance1Descrip = CreateFrame( "ScrollFrame", "Glance1Descrip", mef, "UIPanelScrollFrameTemplate" )
		
		mef.Glance1Descrip.scrollBarHideable = false
		
		mef.Glance1Descrip.Backdrop = CreateFrame("Frame", "Glance1DescripBackdrop", mef)
		mef.Glance1Descrip.Backdrop:SetPoint( "TOPLEFT", Glance1Title, "BOTTOMLEFT", 0, 3)
		mef.Glance1Descrip.Backdrop:SetPoint( "BOTTOMRIGHT", Glance1Title, "BOTTOMRIGHT", -5, -40)
		mef.Glance1Descrip.Backdrop:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance1Descrip:SetPoint( "TOPLEFT", Glance1Title, "BOTTOMLEFT", 0, -5)
		mef.Glance1Descrip:SetPoint( "BOTTOMRIGHT", Glance1Title, "BOTTOMRIGHT", -10, -30)
		mef.Glance1Descrip:EnableMouse(true)

		mef.Glance1Descrip.scrollBarHideable = false

		mef.Glance1Descrip.editbox = CreateFrame( "EditBox", nil, sf )
		mef.Glance1Descrip.editbox:SetPoint( "TOPLEFT", 10, 10 )
		mef.Glance1Descrip.editbox:SetPoint( "BOTTOMRIGHT", 10, 0 )
		mef.Glance1Descrip.editbox:SetHeight( 325 )
		mef.Glance1Descrip.editbox:SetWidth( 280 )
		mef.Glance1Descrip.editbox:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance1Descrip.editbox:EnableMouse(true)
		mef.Glance1Descrip.editbox:SetAutoFocus(false)
		mef.Glance1Descrip.editbox:SetMultiLine(true)
		mef.Glance1Descrip.editbox:SetFontObject( "GameFontHighlight" )
		mef.Glance1Descrip:SetScrollChild( mef.Glance1Descrip.editbox )


		mef.Glance1Descrip.editbox:SetScript( "OnEscapePressed", EditBox_ClearFocus )

		mef.Glance1Descrip.editbox:SetScript( "OnTextChanged", function(self)
			ScrollingEdit_OnTextChanged(self, self:GetParent())
		end	)
		mef.Glance1Descrip.editbox:SetScript( "OnCursorChanged", function(self, x, y, w, h)
			ScrollingEdit_OnCursorChanged(self, x, y-10, w, h)
		end )
		mef.Glance1Descrip.editbox:SetScript( "OnUpdate", function(self, elapsed)
			ScrollingEdit_OnUpdate(self, elapsed, self:GetParent())
		end	)


		ScrollFrame_OnScrollRangeChanged(Glance1Descrip)
		
		-- FontString Title / Details
		
		mef.glance1header = mef:CreateFontString( nil, "OVERLAY", "GameFontNormalSmall" )
		mef.glance1header:SetJustifyH( "LEFT" )
		mef.glance1header:SetJustifyV( "TOP" )
		mef.glance1header:SetPoint( "BOTTOMLEFT", mef.Glance1Title, "TOPLEFT", 3, 1 )
		mef.glance1header:SetText( "Title / Details" )
		
		-- Clear glance button
		mef.Glance1Clear = CreateFrame( "Button", "Glance1ClearButton", mef, uipbt )
		mef.Glance1Clear:SetPoint( "LEFT", mef.Glance1Title, "RIGHT", -5, 2 )
		mef.Glance1Clear:SetText( L["C"] )
		mef.Glance1Clear:SetWidth( 20 )
		mef.Glance1Clear:SetFrameStrata("HIGH")
		mef.Glance1Clear:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Clear this glance and set its icon back to default."], 1.0, 1.0, 1.0 )
		end )
		mef.Glance1Clear:SetScript( "OnLeave", GameTooltip_Hide )
		mef.Glance1Clear:SetScript("OnClick", function (self)
			mrp:ClearGlance(1)
		end )
		
		-------------------------------
		-- Glance 2
		-------------------------------
		
		mef.Glance2Icon = CreateFrame("Frame", "Glance2Icon", mef)
		mef.Glance2Icon:SetPoint( "TOPLEFT", mef.Glance1Icon, "BOTTOMLEFT", 0, -10 )
		mef.Glance2Icon:SetHeight(64)
		mef.Glance2Icon:SetWidth(64)
		local glance2IconTexture = mef.Glance2Icon:CreateTexture("Glance2IconTexture", "ARTWORK")
		glance2IconTexture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		glance2IconTexture:SetAllPoints(mef.Glance2Icon)
		mef.Glance2Icon:SetScript("OnMouseDown", function (self)
			mrp_iconselector_show(mrp_IconCallbackGlance2Icon, mrpSaved.Profiles[mrpSaved.SelectedProfile]["glances"][2]["Icon"] or nil)
		end )
		
		local glance2IconHighlightTexture = mef.Glance2Icon:CreateTexture("Glance2IconHighlightTexture", "OVERLAY")
		glance2IconHighlightTexture:SetTexture("Interface\\AddOns\\MyRolePlay\\Artwork\\GlanceHighlight.blp")
		glance2IconHighlightTexture:SetAllPoints(mef.Glance2Icon)
		glance2IconHighlightTexture:Hide()
		
		mef.Glance2Icon:SetScript("OnEnter", function (self)
			glance2IconHighlightTexture:Show()
		end )
		
		mef.Glance2Icon:SetScript("OnLeave", function (self)
			glance2IconHighlightTexture:Hide()
		end )

		mef.Glance2Title = CreateFrame( "EditBox", "Glance2Title", mef )
		mef.Glance2Title:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance2Title:SetPoint( "TOPLEFT", mef.Glance2Icon, "TOPRIGHT", 3, 0 )
		mef.Glance2Title:SetHeight( 25 )
		mef.Glance2Title:SetWidth( 280 )
		mef.Glance2Title:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance2Title:EnableMouse(true)
		mef.Glance2Title:SetAutoFocus(false)
		mef.Glance2Title:SetMultiLine(false)
		mef.Glance2Title:SetFontObject( "GameFontHighlight" )

		mef.Glance2Title:SetScript( "OnEscapePressed", EditBox_ClearFocus )
		
		mef.Glance2Descrip = CreateFrame( "ScrollFrame", "Glance2Descrip", mef, "UIPanelScrollFrameTemplate" )
		
		mef.Glance2Descrip.scrollBarHideable = false
		
		mef.Glance2Descrip.Backdrop = CreateFrame("Frame", "Glance2DescripBackdrop", mef)
		mef.Glance2Descrip.Backdrop:SetPoint( "TOPLEFT", Glance2Title, "BOTTOMLEFT", 0, 3)
		mef.Glance2Descrip.Backdrop:SetPoint( "BOTTOMRIGHT", Glance2Title, "BOTTOMRIGHT", -5, -40)
		mef.Glance2Descrip.Backdrop:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance2Descrip:SetPoint( "TOPLEFT", Glance2Title, "BOTTOMLEFT", 0, -5)
		mef.Glance2Descrip:SetPoint( "BOTTOMRIGHT", Glance2Title, "BOTTOMRIGHT", -10, -30)
		mef.Glance2Descrip:EnableMouse(true)

		mef.Glance2Descrip.scrollBarHideable = false

		mef.Glance2Descrip.editbox = CreateFrame( "EditBox", nil, sf )
		mef.Glance2Descrip.editbox:SetPoint( "TOPLEFT", 10, 10 )
		mef.Glance2Descrip.editbox:SetPoint( "BOTTOMRIGHT", 10, 0 )
		mef.Glance2Descrip.editbox:SetHeight( 325 )
		mef.Glance2Descrip.editbox:SetWidth( 280 )
		mef.Glance2Descrip.editbox:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance2Descrip.editbox:EnableMouse(true)
		mef.Glance2Descrip.editbox:SetAutoFocus(false)
		mef.Glance2Descrip.editbox:SetMultiLine(true)
		mef.Glance2Descrip.editbox:SetFontObject( "GameFontHighlight" )
		mef.Glance2Descrip:SetScrollChild( mef.Glance2Descrip.editbox )


		mef.Glance2Descrip.editbox:SetScript( "OnEscapePressed", EditBox_ClearFocus )

		mef.Glance2Descrip.editbox:SetScript( "OnTextChanged", function(self)
			ScrollingEdit_OnTextChanged(self, self:GetParent())
		end	)
		mef.Glance2Descrip.editbox:SetScript( "OnCursorChanged", function(self, x, y, w, h)
			ScrollingEdit_OnCursorChanged(self, x, y-10, w, h)
		end )
		mef.Glance2Descrip.editbox:SetScript( "OnUpdate", function(self, elapsed)
			ScrollingEdit_OnUpdate(self, elapsed, self:GetParent())
		end	)


		ScrollFrame_OnScrollRangeChanged(Glance2Descrip)
		
		-- FontString Title / Details
		
		mef.glance2header = mef:CreateFontString( nil, "OVERLAY", "GameFontNormalSmall" )
		mef.glance2header:SetJustifyH( "LEFT" )
		mef.glance2header:SetJustifyV( "TOP" )
		mef.glance2header:SetPoint( "BOTTOMLEFT", mef.Glance2Title, "TOPLEFT", 3, 0 )
		mef.glance2header:SetText( "Title / Details" )
		
		-- Clear glance button
		mef.Glance2Clear = CreateFrame( "Button", "Glance2ClearButton", mef, uipbt )
		mef.Glance2Clear:SetPoint( "LEFT", mef.Glance2Title, "RIGHT", -5, 2 )
		mef.Glance2Clear:SetText( L["C"] )
		mef.Glance2Clear:SetWidth( 20 )
		mef.Glance2Clear:SetFrameStrata("HIGH")
		mef.Glance2Clear:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Clear this glance and set its icon back to default."], 1.0, 1.0, 1.0 )
		end )
		mef.Glance2Clear:SetScript( "OnLeave", GameTooltip_Hide )
		mef.Glance2Clear:SetScript("OnClick", function (self)
			mrp:ClearGlance(2)
		end )
		
		---------------------------------
		-- Glance 3
		---------------------------------
		
		mef.Glance3Icon = CreateFrame("Frame", "Glance3Icon", mef)
		mef.Glance3Icon:SetPoint( "TOPLEFT", mef.Glance2Icon, "BOTTOMLEFT", 0, -10 )
		mef.Glance3Icon:SetHeight(64)
		mef.Glance3Icon:SetWidth(64)
		local glance3IconTexture = mef.Glance3Icon:CreateTexture("Glance3IconTexture", "ARTWORK")
		glance3IconTexture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		glance3IconTexture:SetAllPoints(mef.Glance3Icon)
		mef.Glance3Icon:SetScript("OnMouseDown", function (self)
			mrp_iconselector_show(mrp_IconCallbackGlance3Icon, mrpSaved.Profiles[mrpSaved.SelectedProfile]["glances"][3]["Icon"] or nil)
		end )
		
		local glance3IconHighlightTexture = mef.Glance3Icon:CreateTexture("Glance3IconHighlightTexture", "OVERLAY")
		glance3IconHighlightTexture:SetTexture("Interface\\AddOns\\MyRolePlay\\Artwork\\GlanceHighlight.blp")
		glance3IconHighlightTexture:SetAllPoints(mef.Glance3Icon)
		glance3IconHighlightTexture:Hide()
		
		mef.Glance3Icon:SetScript("OnEnter", function (self)
			glance3IconHighlightTexture:Show()
		end )
		
		mef.Glance3Icon:SetScript("OnLeave", function (self)
			glance3IconHighlightTexture:Hide()
		end )

		mef.Glance3Title = CreateFrame( "EditBox", "Glance3Title", mef )
		mef.Glance3Title:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance3Title:SetPoint( "TOPLEFT", mef.Glance3Icon, "TOPRIGHT", 3, 0 )
		mef.Glance3Title:SetHeight( 25 )
		mef.Glance3Title:SetWidth( 280 )
		mef.Glance3Title:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance3Title:EnableMouse(true)
		mef.Glance3Title:SetAutoFocus(false)
		mef.Glance3Title:SetMultiLine(false)
		mef.Glance3Title:SetFontObject( "GameFontHighlight" )

		mef.Glance3Title:SetScript( "OnEscapePressed", EditBox_ClearFocus )
		
		mef.Glance3Descrip = CreateFrame( "ScrollFrame", "Glance3Descrip", mef, "UIPanelScrollFrameTemplate" )
		
		mef.Glance3Descrip.scrollBarHideable = false
		
		mef.Glance3Descrip.Backdrop = CreateFrame("Frame", "Glance3DescripBackdrop", mef)
		mef.Glance3Descrip.Backdrop:SetPoint( "TOPLEFT", Glance3Title, "BOTTOMLEFT", 0, 3)
		mef.Glance3Descrip.Backdrop:SetPoint( "BOTTOMRIGHT", Glance3Title, "BOTTOMRIGHT", -5, -40)
		mef.Glance3Descrip.Backdrop:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance3Descrip:SetPoint( "TOPLEFT", Glance3Title, "BOTTOMLEFT", 0, -5)
		mef.Glance3Descrip:SetPoint( "BOTTOMRIGHT", Glance3Title, "BOTTOMRIGHT", -10, -30)
		mef.Glance3Descrip:EnableMouse(true)

		mef.Glance3Descrip.scrollBarHideable = false

		mef.Glance3Descrip.editbox = CreateFrame( "EditBox", nil, sf )
		mef.Glance3Descrip.editbox:SetPoint( "TOPLEFT", 10, 10 )
		mef.Glance3Descrip.editbox:SetPoint( "BOTTOMRIGHT", 10, 0 )
		mef.Glance3Descrip.editbox:SetHeight( 325 )
		mef.Glance3Descrip.editbox:SetWidth( 280 )
		mef.Glance3Descrip.editbox:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance3Descrip.editbox:EnableMouse(true)
		mef.Glance3Descrip.editbox:SetAutoFocus(false)
		mef.Glance3Descrip.editbox:SetMultiLine(true)
		mef.Glance3Descrip.editbox:SetFontObject( "GameFontHighlight" )
		mef.Glance3Descrip:SetScrollChild( mef.Glance3Descrip.editbox )


		mef.Glance3Descrip.editbox:SetScript( "OnEscapePressed", EditBox_ClearFocus )

		mef.Glance3Descrip.editbox:SetScript( "OnTextChanged", function(self)
			ScrollingEdit_OnTextChanged(self, self:GetParent())
		end	)
		mef.Glance3Descrip.editbox:SetScript( "OnCursorChanged", function(self, x, y, w, h)
			ScrollingEdit_OnCursorChanged(self, x, y-10, w, h)
		end )
		mef.Glance3Descrip.editbox:SetScript( "OnUpdate", function(self, elapsed)
			ScrollingEdit_OnUpdate(self, elapsed, self:GetParent())
		end	)


		ScrollFrame_OnScrollRangeChanged(Glance3Descrip)
		
		-- FontString Title / Details
		
		mef.glance3header = mef:CreateFontString( nil, "OVERLAY", "GameFontNormalSmall" )
		mef.glance3header:SetJustifyH( "LEFT" )
		mef.glance3header:SetJustifyV( "TOP" )
		mef.glance3header:SetPoint( "BOTTOMLEFT", mef.Glance3Title, "TOPLEFT", 3, 0 )
		mef.glance3header:SetText( "Title / Details" )
		
		-- Clear glance button
		mef.Glance3Clear = CreateFrame( "Button", "Glance3ClearButton", mef, uipbt )
		mef.Glance3Clear:SetPoint( "LEFT", mef.Glance3Title, "RIGHT", -5, 2 )
		mef.Glance3Clear:SetText( L["C"] )
		mef.Glance3Clear:SetWidth( 20 )
		mef.Glance3Clear:SetFrameStrata("HIGH")
		mef.Glance3Clear:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Clear this glance and set its icon back to default."], 1.0, 1.0, 1.0 )
		end )
		mef.Glance3Clear:SetScript( "OnLeave", GameTooltip_Hide )
		mef.Glance3Clear:SetScript("OnClick", function (self)
			mrp:ClearGlance(3)
		end )
		
		------------------------------------
		-- Glance 4
		------------------------------------
		
		mef.Glance4Icon = CreateFrame("Frame", "Glance4Icon", mef)
		mef.Glance4Icon:SetPoint( "TOPLEFT", mef.Glance3Icon, "BOTTOMLEFT", 0, -10 )
		mef.Glance4Icon:SetHeight(64)
		mef.Glance4Icon:SetWidth(64)
		local glance4IconTexture = mef.Glance4Icon:CreateTexture("Glance4IconTexture", "ARTWORK")
		glance4IconTexture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		glance4IconTexture:SetAllPoints(mef.Glance4Icon)
		mef.Glance4Icon:SetScript("OnMouseDown", function (self)
			mrp_iconselector_show(mrp_IconCallbackGlance4Icon, mrpSaved.Profiles[mrpSaved.SelectedProfile]["glances"][4]["Icon"] or nil)
		end )
		
		local glance4IconHighlightTexture = mef.Glance4Icon:CreateTexture("Glance4IconHighlightTexture", "OVERLAY")
		glance4IconHighlightTexture:SetTexture("Interface\\AddOns\\MyRolePlay\\Artwork\\GlanceHighlight.blp")
		glance4IconHighlightTexture:SetAllPoints(mef.Glance4Icon)
		glance4IconHighlightTexture:Hide()
		
		mef.Glance4Icon:SetScript("OnEnter", function (self)
			glance4IconHighlightTexture:Show()
		end )
		
		mef.Glance4Icon:SetScript("OnLeave", function (self)
			glance4IconHighlightTexture:Hide()
		end )

		mef.Glance4Title = CreateFrame( "EditBox", "Glance4Title", mef )
		mef.Glance4Title:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance4Title:SetPoint( "TOPLEFT", mef.Glance4Icon, "TOPRIGHT", 3, 0 )
		mef.Glance4Title:SetHeight( 25 )
		mef.Glance4Title:SetWidth( 280 )
		mef.Glance4Title:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance4Title:EnableMouse(true)
		mef.Glance4Title:SetAutoFocus(false)
		mef.Glance4Title:SetMultiLine(false)
		mef.Glance4Title:SetFontObject( "GameFontHighlight" )

		mef.Glance4Title:SetScript( "OnEscapePressed", EditBox_ClearFocus )
		
		mef.Glance4Descrip = CreateFrame( "ScrollFrame", "Glance4Descrip", mef, "UIPanelScrollFrameTemplate" )
		
		mef.Glance4Descrip.scrollBarHideable = false
		
		mef.Glance4Descrip.Backdrop = CreateFrame("Frame", "Glance4DescripBackdrop", mef)
		mef.Glance4Descrip.Backdrop:SetPoint( "TOPLEFT", Glance4Title, "BOTTOMLEFT", 0, 3)
		mef.Glance4Descrip.Backdrop:SetPoint( "BOTTOMRIGHT", Glance4Title, "BOTTOMRIGHT", -5, -40)
		mef.Glance4Descrip.Backdrop:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance4Descrip:SetPoint( "TOPLEFT", Glance4Title, "BOTTOMLEFT", 0, -5)
		mef.Glance4Descrip:SetPoint( "BOTTOMRIGHT", Glance4Title, "BOTTOMRIGHT", -10, -30)
		mef.Glance4Descrip:EnableMouse(true)

		mef.Glance4Descrip.scrollBarHideable = false

		mef.Glance4Descrip.editbox = CreateFrame( "EditBox", nil, sf )
		mef.Glance4Descrip.editbox:SetPoint( "TOPLEFT", 10, 10 )
		mef.Glance4Descrip.editbox:SetPoint( "BOTTOMRIGHT", 10, 0 )
		mef.Glance4Descrip.editbox:SetHeight( 325 )
		mef.Glance4Descrip.editbox:SetWidth( 280 )
		mef.Glance4Descrip.editbox:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance4Descrip.editbox:EnableMouse(true)
		mef.Glance4Descrip.editbox:SetAutoFocus(false)
		mef.Glance4Descrip.editbox:SetMultiLine(true)
		mef.Glance4Descrip.editbox:SetFontObject( "GameFontHighlight" )
		mef.Glance4Descrip:SetScrollChild( mef.Glance4Descrip.editbox )


		mef.Glance4Descrip.editbox:SetScript( "OnEscapePressed", EditBox_ClearFocus )

		mef.Glance4Descrip.editbox:SetScript( "OnTextChanged", function(self)
			ScrollingEdit_OnTextChanged(self, self:GetParent())
		end	)
		mef.Glance4Descrip.editbox:SetScript( "OnCursorChanged", function(self, x, y, w, h)
			ScrollingEdit_OnCursorChanged(self, x, y-10, w, h)
		end )
		mef.Glance4Descrip.editbox:SetScript( "OnUpdate", function(self, elapsed)
			ScrollingEdit_OnUpdate(self, elapsed, self:GetParent())
		end	)


		ScrollFrame_OnScrollRangeChanged(Glance4Descrip)
		
		-- FontString Title / Details
		
		mef.glance4header = mef:CreateFontString( nil, "OVERLAY", "GameFontNormalSmall" )
		mef.glance4header:SetJustifyH( "LEFT" )
		mef.glance4header:SetJustifyV( "TOP" )
		mef.glance4header:SetPoint( "BOTTOMLEFT", mef.Glance4Title, "TOPLEFT", 3, 0 )
		mef.glance4header:SetText( "Title / Details" )
		
		-- Clear glance button
		mef.Glance4Clear = CreateFrame( "Button", "Glance4ClearButton", mef, uipbt )
		mef.Glance4Clear:SetPoint( "LEFT", mef.Glance4Title, "RIGHT", -5, 2 )
		mef.Glance4Clear:SetText( L["C"] )
		mef.Glance4Clear:SetWidth( 20 )
		mef.Glance4Clear:SetFrameStrata("HIGH")
		mef.Glance4Clear:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Clear this glance and set its icon back to default."], 1.0, 1.0, 1.0 )
		end )
		mef.Glance4Clear:SetScript( "OnLeave", GameTooltip_Hide )
		mef.Glance4Clear:SetScript("OnClick", function (self)
			mrp:ClearGlance(4)
		end )
		
		--------------------------------
		-- Glance 5
		--------------------------------
		
		mef.Glance5Icon = CreateFrame("Frame", "Glance5Icon", mef)
		mef.Glance5Icon:SetPoint( "TOPLEFT", mef.Glance4Icon, "BOTTOMLEFT", 0, -10 )
		mef.Glance5Icon:SetHeight(64)
		mef.Glance5Icon:SetWidth(64)
		local glance5IconTexture = mef.Glance3Icon:CreateTexture("Glance5IconTexture", "ARTWORK")
		glance5IconTexture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		glance5IconTexture:SetAllPoints(mef.Glance5Icon)
		mef.Glance5Icon:SetScript("OnMouseDown", function (self)
			mrp_iconselector_show(mrp_IconCallbackGlance5Icon, mrpSaved.Profiles[mrpSaved.SelectedProfile]["glances"][5]["Icon"] or nil)
		end )
		
		local glance5IconHighlightTexture = mef.Glance5Icon:CreateTexture("Glance5IconHighlightTexture", "OVERLAY")
		glance5IconHighlightTexture:SetTexture("Interface\\AddOns\\MyRolePlay\\Artwork\\GlanceHighlight.blp")
		glance5IconHighlightTexture:SetAllPoints(mef.Glance5Icon)
		glance5IconHighlightTexture:Hide()
		
		mef.Glance5Icon:SetScript("OnEnter", function (self)
			glance5IconHighlightTexture:Show()
		end )
		
		mef.Glance5Icon:SetScript("OnLeave", function (self)
			glance5IconHighlightTexture:Hide()
		end )

		mef.Glance5Title = CreateFrame( "EditBox", "Glance5Title", mef )
		mef.Glance5Title:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance5Title:SetPoint( "TOPLEFT", mef.Glance5Icon, "TOPRIGHT", 3, 0 )
		mef.Glance5Title:SetHeight( 25 )
		mef.Glance5Title:SetWidth( 280 )
		mef.Glance5Title:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance5Title:EnableMouse(true)
		mef.Glance5Title:SetAutoFocus(false)
		mef.Glance5Title:SetMultiLine(false)
		mef.Glance5Title:SetFontObject( "GameFontHighlight" )

		mef.Glance5Title:SetScript( "OnEscapePressed", EditBox_ClearFocus )
		
		mef.Glance5Descrip = CreateFrame( "ScrollFrame", "Glance5Descrip", mef, "UIPanelScrollFrameTemplate" )
		
		mef.Glance5Descrip.scrollBarHideable = false
		
		mef.Glance5Descrip.Backdrop = CreateFrame("Frame", "Glance5DescripBackdrop", mef)
		mef.Glance5Descrip.Backdrop:SetPoint( "TOPLEFT", Glance5Title, "BOTTOMLEFT", 0, 3)
		mef.Glance5Descrip.Backdrop:SetPoint( "BOTTOMRIGHT", Glance5Title, "BOTTOMRIGHT", -5, -40)
		mef.Glance5Descrip.Backdrop:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 12,
				insets = { left = 5, right = 3, top = 3, bottom = 3	},
		} )
		mef.Glance5Descrip:SetPoint( "TOPLEFT", Glance5Title, "BOTTOMLEFT", 0, -5)
		mef.Glance5Descrip:SetPoint( "BOTTOMRIGHT", Glance5Title, "BOTTOMRIGHT", -10, -30)
		mef.Glance5Descrip:EnableMouse(true)

		mef.Glance5Descrip.scrollBarHideable = false

		mef.Glance5Descrip.editbox = CreateFrame( "EditBox", nil, sf )
		mef.Glance5Descrip.editbox:SetPoint( "TOPLEFT", 10, 10 )
		mef.Glance5Descrip.editbox:SetPoint( "BOTTOMRIGHT", 10, 0 )
		mef.Glance5Descrip.editbox:SetHeight( 325 )
		mef.Glance5Descrip.editbox:SetWidth( 280 )
		mef.Glance5Descrip.editbox:SetTextInsets( 7, 7, 0, 0 )
		mef.Glance5Descrip.editbox:EnableMouse(true)
		mef.Glance5Descrip.editbox:SetAutoFocus(false)
		mef.Glance5Descrip.editbox:SetMultiLine(true)
		mef.Glance5Descrip.editbox:SetFontObject( "GameFontHighlight" )
		mef.Glance5Descrip:SetScrollChild( mef.Glance5Descrip.editbox )


		mef.Glance5Descrip.editbox:SetScript( "OnEscapePressed", EditBox_ClearFocus )

		mef.Glance5Descrip.editbox:SetScript( "OnTextChanged", function(self)
			ScrollingEdit_OnTextChanged(self, self:GetParent())
		end	)
		mef.Glance5Descrip.editbox:SetScript( "OnCursorChanged", function(self, x, y, w, h)
			ScrollingEdit_OnCursorChanged(self, x, y-10, w, h)
		end )
		mef.Glance5Descrip.editbox:SetScript( "OnUpdate", function(self, elapsed)
			ScrollingEdit_OnUpdate(self, elapsed, self:GetParent())
		end	)


		ScrollFrame_OnScrollRangeChanged(Glance5Descrip)
		
		-- FontString Title / Details
		
		mef.glance5header = mef:CreateFontString( nil, "OVERLAY", "GameFontNormalSmall" )
		mef.glance5header:SetJustifyH( "LEFT" )
		mef.glance5header:SetJustifyV( "TOP" )
		mef.glance5header:SetPoint( "BOTTOMLEFT", mef.Glance5Title, "TOPLEFT", 3, 0 )
		mef.glance5header:SetText( "Title / Details" )
		
		-- Clear glance button
		mef.Glance5Clear = CreateFrame( "Button", "Glance5ClearButton", mef, uipbt )
		mef.Glance5Clear:SetPoint( "LEFT", mef.Glance5Title, "RIGHT", -5, 2 )
		mef.Glance5Clear:SetText( L["C"] )
		mef.Glance5Clear:SetWidth( 20 )
		mef.Glance5Clear:SetFrameStrata("HIGH")
		mef.Glance5Clear:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Clear this glance and set its icon back to default."], 1.0, 1.0, 1.0 )
		end )
		mef.Glance5Clear:SetScript( "OnLeave", GameTooltip_Hide )
		mef.Glance5Clear:SetScript("OnClick", function (self)
			mrp:ClearGlance(5)
		end )
		
		mef.ok = CreateFrame( "Button", "MyRolePlayGlanceEditFrameOK", mef, uipbt )
		mef.ok:SetPoint( "BOTTOMRIGHT", CharacterFrame, "BOTTOMRIGHT", -8, 7 )
		mef.ok:SetText( "Save" )
		mef.ok:SetWidth( 90 )
		mef.ok:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( "Save changes.", 1.0, 1.0, 1.0 )
		end )
		mef.ok:SetScript( "OnLeave", GameTooltip_Hide )
		mef.ok:SetScript("OnClick", function (self)
			local field = MyRolePlayGlanceEditFrame.field
			local profile = mrpSaved.SelectedProfile
			if(type(mrpSaved.Profiles[profile]["glances"]) ~= "table") then -- If they don't have glances in their profile, make it.
				mrpSaved.Profiles[profile]["glances"] = {}
				for i = 1, 5, 1 do
					mrpSaved.Profiles[profile]["glances"][i] = {}
				end
			end
			for i = 1, 5, 1 do
				mrpSaved.Profiles[profile]["glances"][i]["Title"] = _G["Glance" .. i .. "Title"]:GetText()
				mrpSaved.Profiles[profile]["glances"][i]["Description"] = _G["Glance" .. i .. "Descrip"].editbox:GetText()
				--mrpSaved.Profiles[profile]["glances"][i]["Icon"] = _G["Glance" .. i .. "IconTexture"]:GetTexture()
			end
			-- Setup MSP readable glances
			local glanceMSP = ""
			local checkTitle
			local first = true
			for i = 1, 5, 1 do
				checkTitle = string.trim(mrpSaved.Profiles[profile]["glances"][i]["Title"])
				if(checkTitle ~= "" and checkTitle ~= nil) then
					if(first == true) then
						glanceMSP = "|T" .. mrpSaved.Profiles[profile]["glances"][i]["Icon"] .. ":32:32|t\n#" .. mrpSaved.Profiles[profile]["glances"][i]["Title"] .. "\n\n" .. mrpSaved.Profiles[profile]["glances"][i]["Description"]
						first = false
					else
						glanceMSP = glanceMSP .. "\n\n---\n\n" .. "|T" .. mrpSaved.Profiles[profile]["glances"][i]["Icon"] .. ":32:32|t\n#" .. mrpSaved.Profiles[profile]["glances"][i]["Title"] .. "\n\n" .. mrpSaved.Profiles[profile]["glances"][i]["Description"]
					end
				end
			end
			glanceMSP = glanceMSP:gsub("|TINTERFACE\\ICONS", "|TInterface\\Icons")
			mrp:SaveField('PE', glanceMSP)
			MyRolePlayGlanceEditFrame:Hide()
		end )

		mef.cancel = CreateFrame( "Button", "MyRolePlayGlanceEditFrameCancel", mef, uipbt )
		mef.cancel:SetPoint( "RIGHT", mef.ok, "LEFT", -16, 0 )
		mef.cancel:SetText( "Cancel" )
		mef.cancel:SetWidth( 90 )
		mef.cancel:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( "Cancel changes and return to the way it was.", 1.0, 1.0, 1.0 )
		end )
		mef.cancel:SetScript( "OnLeave", GameTooltip_Hide )
		mef.cancel:SetScript("OnClick", function (self)
			for i = 1, 5, 1 do -- Restore the previous icons if they cancel instead of save.
				mrpSaved.Profiles[mrpSaved.SelectedProfile]["glances"][i]["Icon"] = previousGlanceIcons[i]
			end
			MyRolePlayGlanceEditFrame:Hide()
		end )
	end
	-- MyRolePlayMultiEditFrame
	if not MyRolePlayMultiEditFrame then
		local mef = CreateFrame( "Frame", "MyRolePlayMultiEditFrame", MyRolePlayCharacterFrame, "InsetFrameTemplate" )
		mef:Hide()
		mef:SetPoint( "TOPLEFT", MyRolePlayCharacterFrame, "TOPLEFT", 332, -61 )
		mef:SetPoint( "BOTTOM", CharacterFrameInset, "BOTTOM", 0, 30 )
		mef:SetPoint( "RIGHT", CharacterFrame, "RIGHT", -8, 0 )
		mef:EnableDrawLayer("ARTWORK")

		mef:SetScript("OnShow", function(self)
			CharacterFrame:SetWidth( 700 )
			CharacterFrame.Expanded = true
			UpdateUIPanelPositions(CharacterFrame)
			MyRolePlayMultiEditFrame.sf.editbox:SetFocus()
		end	)
		mef:SetScript("OnHide", function(self)
			CharacterFrame:SetWidth( PANEL_DEFAULT_WIDTH )
			CharacterFrame.Expanded = false
			UpdateUIPanelPositions(CharacterFrame)
		end	)
		mef.title = mef:CreateFontString( nil, "ARTWORK", "GameFontNormal" )
		mef.title:SetPoint( "TOP", mef, "TOP", 0, 27 )

		mef:EnableDrawLayer("OVERLAY")
		mef.sf = CreateFrame( "ScrollFrame", "MyRolePlayMultiEditFrameScrollFrame", mef, "UIPanelScrollFrameTemplate2" )
		mef.sf:SetPoint( "TOPLEFT", 8, -6 )
		mef.sf:SetPoint( "BOTTOMRIGHT", -28, 6 )
		mef.sf:SetSize( 325, 325 )

		mef.sf.scrollBarHideable = false

		mef.sf.editbox = CreateFrame( "EditBox", nil, mef.sf )
		mef.sf.editbox:SetPoint( "TOPLEFT" )
		mef.sf.editbox:SetPoint( "BOTTOMLEFT" )
		mef.sf.editbox:SetHeight( 325 )
		mef.sf.editbox:SetWidth( 325 )
		mef.sf.editbox:SetTextInsets( 5, 5, 3, 3 )
		mef.sf.editbox:EnableMouse(true)
		mef.sf.editbox:SetAutoFocus(false)
		mef.sf.editbox:SetMultiLine(true)
		mef.sf.editbox:SetFontObject( "GameFontHighlight" )
		mef.sf:SetScrollChild( mef.sf.editbox )


		mef.sf.editbox:SetScript( "OnEscapePressed", EditBox_ClearFocus )

		mef.sf.editbox:SetScript( "OnTextChanged", function(self)
			ScrollingEdit_OnTextChanged(self, self:GetParent())
		end	)
		mef.sf.editbox:SetScript( "OnCursorChanged", function(self, x, y, w, h)
			ScrollingEdit_OnCursorChanged(self, x, y-10, w, h)
		end )
		mef.sf.editbox:SetScript( "OnUpdate", function(self, elapsed)
			ScrollingEdit_OnUpdate(self, elapsed, self:GetParent())
		end	)


		ScrollFrame_OnScrollRangeChanged(MyRolePlayMultiEditFrameScrollFrame)
		
		-- Descrip colour button
		mef.cdb = CreateFrame( "Button", "MyRolePlayMultiEditFrame_DescripColourButton", mef, uipbt )
		local mrpColourButtonTexture = mef.cdb:CreateTexture("MyRolePlay_DescripColourButtonBackground", "BACKGROUND")
		mrpColourButtonTexture:SetAllPoints(mef.cdb)
		mrpColourButtonTexture:SetTexture(0.5,0.5,1)
		mef.cdb:SetPoint( "LEFT", mef.title, "LEFT", -130, 0 )
		mef.cdb:SetText( L["Change Colour"] )
		mef.cdb:SetWidth( 120 )
		mef.cdb:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Add a colour to your description."], 1.0, 1.0, 1.0 )
		end )
		mef.cdb:SetScript( "OnLeave", GameTooltip_Hide )
		mef.cdb:SetScript("OnClick", function (self)
		local savedR, savedG, savedB
			if(mrpSaved.Profiles[mrpSaved.SelectedProfile]["descripColour"]) then
				savedR = mrpSaved.Profiles[mrpSaved.SelectedProfile]["descripColour"]["r"]
				savedG = mrpSaved.Profiles[mrpSaved.SelectedProfile]["descripColour"]["g"]
				savedB = mrpSaved.Profiles[mrpSaved.SelectedProfile]["descripColour"]["b"]
			elseif(mrpSaved.Profiles["Default"]["descripColour"]) then
				savedR = mrpSaved.Profiles["Default"]["descripColour"]["r"]
				savedG = mrpSaved.Profiles["Default"]["descripColour"]["g"]
				savedB = mrpSaved.Profiles["Default"]["descripColour"]["b"]
			else
				savedR, savedG, savedB = 0.9924675, 0.9924675, 0.9924675 -- Magic values :3
			end
			mrp_ShowColourPicker(savedR, savedG, savedB, 1, mrp_DescripColourChangedCallback)
		end )
		
		-- Restore descrip colour to default button
		mef.cdr = CreateFrame( "Button", "MyRolePlayMultiEditFrame_RestoreDefaultColourButtonDescrip", mef, uipbt )
		local mrpColourButtonTexture = mef.cdr:CreateTexture("MyRolePlay_DescripColourRestoreButtonBackground", "BACKGROUND")
		mrpColourButtonTexture:SetAllPoints(mef.cdr)
		mrpColourButtonTexture:SetTexture(0.5,0.5,1)
		mef.cdr:SetPoint( "RIGHT", mef.title, "RIGHT", 130, 0)
		mef.cdr:SetText( L["Restore Colour"] )
		mef.cdr:SetWidth( 120 )
		mef.cdr:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Restore your colour to the default profile description's colour."], 1.0, 1.0, 1.0 )
		end )
		mef.cdr:SetScript( "OnLeave", GameTooltip_Hide )
		mef.cdr:SetScript("OnClick", function (self)
			mrpSaved.Profiles[mrpSaved.SelectedProfile]["descripColour"] = nil
			ColorPickerFrame:Hide()
			if(msp.my["DE"]) then
				msp.my["DE"] = string.gsub(msp.my["DE"], "|cff%x%x%x%x%x%x", "")
			end
			if(mrpSaved.Profiles[mrpSaved.SelectedProfile]["DE"]) then
				mrpSaved.Profiles[mrpSaved.SelectedProfile]["DE"] = string.gsub(mrpSaved.Profiles[mrpSaved.SelectedProfile]["DE"], "|cff%x%x%x%x%x%x", "")
			end
			if(mef.sf.editbox) then
				local currentText = mef.sf.editbox:GetText()
				currentText = string.gsub(currentText, "|cff%x%x%x%x%x%x", "")
				mef.sf.editbox:SetText(currentText)
			end
			mrp:UpdateColour("DE")
			mrp:UpdateCharacterFrame()
		end )
		
		-- End descrip default colour button code
		
		-- History colour button
		mef.chb = CreateFrame( "Button", "MyRolePlayMultiEditFrame_HistoryColourButton", mef, uipbt )
		local mrpColourButtonTexture = mef.chb:CreateTexture("MyRolePlay_HistoryColourButtonBackground", "BACKGROUND")
		mrpColourButtonTexture:SetAllPoints(mef.chb)
		mrpColourButtonTexture:SetTexture(0.5,0.5,1)
		mef.chb:SetPoint( "LEFT", mef.title, "LEFT", -130, 0 )
		mef.chb:SetText( L["Change Colour"] )
		mef.chb:SetWidth( 120 )
		mef.chb:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Add a colour to your history."], 1.0, 1.0, 1.0 )
		end )
		mef.chb:SetScript( "OnLeave", GameTooltip_Hide )
		mef.chb:SetScript("OnClick", function (self)
		local savedR, savedG, savedB
			if(mrpSaved.Profiles[mrpSaved.SelectedProfile]["historyColour"]) then
				savedR = mrpSaved.Profiles[mrpSaved.SelectedProfile]["historyColour"]["r"]
				savedG = mrpSaved.Profiles[mrpSaved.SelectedProfile]["historyColour"]["g"]
				savedB = mrpSaved.Profiles[mrpSaved.SelectedProfile]["historyColour"]["b"]
			elseif(mrpSaved.Profiles["Default"]["historyColour"]) then
				savedR = mrpSaved.Profiles["Default"]["historyColour"]["r"]
				savedG = mrpSaved.Profiles["Default"]["historyColour"]["g"]
				savedB = mrpSaved.Profiles["Default"]["historyColour"]["b"]
			else
				savedR, savedG, savedB = 0.9924675, 0.9924675, 0.9924675 -- Magic values :3
			end
			mrp_ShowColourPicker(savedR, savedG, savedB, 1, mrp_HistoryColourChangedCallback)
		end )
		
		-- Restore history colour to default button
		mef.chr = CreateFrame( "Button", "MyRolePlayMultiEditFrame_RestoreDefaultColourButtonHistory", mef, uipbt )
		local mrpColourButtonTexture = mef.chr:CreateTexture("MyRolePlay_HistoryColourRestoreButtonBackground", "BACKGROUND")
		mrpColourButtonTexture:SetAllPoints(mef.chr)
		mrpColourButtonTexture:SetTexture(0.5,0.5,1)
		mef.chr:SetPoint( "RIGHT", mef.title, "RIGHT", 130, 0)
		mef.chr:SetText( L["Restore Colour"] )
		mef.chr:SetWidth( 120 )
		mef.chr:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Restore your colour to the default profile history colour."], 1.0, 1.0, 1.0 )
		end )
		mef.chr:SetScript( "OnLeave", GameTooltip_Hide )
		mef.chr:SetScript("OnClick", function (self)
			mrpSaved.Profiles[mrpSaved.SelectedProfile]["historyColour"] = nil
			ColorPickerFrame:Hide()
			if(msp.my["HI"]) then
				msp.my["HI"] = string.gsub(msp.my["HI"], "|cff%x%x%x%x%x%x", "")
			end
			if(mrpSaved.Profiles[mrpSaved.SelectedProfile]["HI"]) then
				mrpSaved.Profiles[mrpSaved.SelectedProfile]["HI"] = string.gsub(mrpSaved.Profiles[mrpSaved.SelectedProfile]["HI"], "|cff%x%x%x%x%x%x", "")
			end
			if(mef.sf.editbox) then
				local currentText = mef.sf.editbox:GetText()
				currentText = string.gsub(currentText, "|cff%x%x%x%x%x%x", "")
				mef.sf.editbox:SetText(currentText)
			end
			mrp:UpdateColour("HI")
			mrp:UpdateCharacterFrame()
		end )
		
		-- End descrip default colour button code

		mef.ok = CreateFrame( "Button", "MyRolePlayMultiEditFrameOK", mef, uipbt )
		mef.ok:SetPoint( "BOTTOMRIGHT", CharacterFrame, "BOTTOMRIGHT", -8, 7 )
		mef.ok:SetText( L["Save"] )
		mef.ok:SetWidth( 90 )
		mef.ok:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Save the changes you’ve made back to the profile."], 1.0, 1.0, 1.0 )
		end )
		mef.ok:SetScript( "OnLeave", GameTooltip_Hide )
		mef.ok:SetScript("OnClick", function (self)
			local field = MyRolePlayMultiEditFrame.field
			local newtext = MyRolePlayMultiEditFrame.sf.editbox:GetText()
			mrp:SaveField( field, newtext )
			MyRolePlayMultiEditFrame:Hide()
		end )

		mef.cancel = CreateFrame( "Button", "MyRolePlayMultiEditFrameCancel", mef, uipbt )
		mef.cancel:SetPoint( "RIGHT", mef.ok, "LEFT", -16, 0 )
		mef.cancel:SetText( L["Cancel"] )
		mef.cancel:SetWidth( 90 )
		mef.cancel:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Cancel any changes and return to the way it was."], 1.0, 1.0, 1.0 )
		end )
		mef.cancel:SetScript( "OnLeave", GameTooltip_Hide )
		mef.cancel:SetScript("OnClick", function (self)
			-- we, uh, don't need to do anything
			MyRolePlayMultiEditFrame:Hide()
		end )

		mef.inherit = CreateFrame( "Button", "MyRolePlayMultiEditFrameInherit", mef, uipbt )
		mef.inherit:SetPoint( "LEFT", mef )
		mef.inherit:SetPoint( "TOP", mef.ok )
		mef.inherit:SetText( L["Inherit"] )
		mef.inherit:SetWidth( 90 )
		mef.inherit:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Cancel any changes, and use the same contents as the Default profile."], 1.0, 1.0, 1.0 )
		end )
		mef.inherit:SetScript( "OnLeave", GameTooltip_Hide )
		mef.inherit:SetScript("OnClick", function (self)
			local field = MyRolePlayMultiEditFrame.field
			mrp:SaveField( field, nil )
			MyRolePlayMultiEditFrame:Hide()
		end )
		mef.inherit:Hide()

		mef.inherited = mef:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
		mef.inherited:SetJustifyH( "LEFT" )
		mef.inherited:SetPoint( "LEFT", mef, "LEFT", 16, 0 )
		mef.inherited:SetPoint( "BOTTOMLEFT", mef, "BOTTOM", 0, -22 )
		mef.inherited:SetText( L["Inherited from Default."] )
		mef.inherited:Hide()
	end

	-- MyRolePlayEditFrame
	if not MyRolePlayEditFrame then
		local mef = CreateFrame( "Frame", "MyRolePlayEditFrame", MyRolePlayCharacterFrame )
		mef:Hide()
		mef:SetPoint( "TOPLEFT", MyRolePlayCharacterFrame, "TOPLEFT", 332, -61 )
		mef:SetPoint( "BOTTOM", CharacterFrameInset, "BOTTOM", 0, 30 )
		mef:SetPoint( "RIGHT", CharacterFrame, "RIGHT", -8, 0 )
		mef:EnableDrawLayer("ARTWORK")

		mef:SetScript("OnShow", function(self)
			CharacterFrame:SetWidth( 700 )
			CharacterFrame.Expanded = true
			UpdateUIPanelPositions(CharacterFrame)
			MyRolePlayEditFrame.editbox:SetFocus()
		end	)
		mef:SetScript("OnHide", function(self)
			CharacterFrame:SetWidth( PANEL_DEFAULT_WIDTH )
			CharacterFrame.Expanded = false
			UpdateUIPanelPositions(CharacterFrame)
		end	)
		mef.title = mef:CreateFontString( nil, "ARTWORK", "GameFontNormal" )
		mef.title:SetPoint( "TOP", mef, "TOP", 0, 27 )

		mef.desc = mef:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
		mef.desc:SetWordWrap(true)
		mef.desc:SetJustifyH( "LEFT" )
		mef.desc:SetJustifyV( "TOP" )
		mef.desc:SetPoint( "TOP", mef, "TOP", 0, -10 )


		mef:EnableDrawLayer("OVERLAY")

		mef.editbox = CreateFrame( "EditBox", nil, mef )
		mef.editbox:SetBackdrop(	{
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
				edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
				tile = true,
				tileSize = 32,
				edgeSize = 32,
				insets = { left = 11, right = 12, top = 12, bottom = 11	},
		} )
		mef.editbox:SetPoint( "CENTER" )
		mef.editbox:SetPoint( "LEFT", 5 )
		mef.editbox:SetPoint( "RIGHT", 5 )
		mef.editbox:SetHeight( 40 )
		mef.editbox:SetWidth( 200 )
		mef.editbox:SetTextInsets( 12, 12, 3, 3 )
		mef.editbox:EnableMouse(true)
		mef.editbox:SetAutoFocus(false)
		mef.editbox:SetMultiLine(false)
		mef.editbox:SetFontObject( "GameFontHighlight" )

		mef.editbox:SetScript( "OnEscapePressed", EditBox_ClearFocus )
		
		-- Name colour button
		mef.cpb = CreateFrame( "Button", "MyRolePlayCharacterFrame_ColourButton", mef, uipbt )
		local mrpColourButtonTexture = mef.cpb:CreateTexture("MyRolePlay_ColourButtonBackground", "BACKGROUND")
		mrpColourButtonTexture:SetAllPoints(mef.cpb)
		mrpColourButtonTexture:SetTexture(0.5,0.5,1)
		mef.cpb:SetPoint( "BOTTOMLEFT", mef.editbox, "BOTTOMLEFT", 20, -50 )
		mef.cpb:SetText( L["Change Name Colour"] )
		mef.cpb:SetWidth( 150 )
		mef.cpb:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Add a colour to your name."], 1.0, 1.0, 1.0 )
		end )
		mef.cpb:SetScript( "OnLeave", GameTooltip_Hide )
		mef.cpb:SetScript("OnClick", function (self)
		local savedR, savedG, savedB
			if(mrpSaved.Profiles[mrpSaved.SelectedProfile]["nameColour"]) then
				savedR = mrpSaved.Profiles[mrpSaved.SelectedProfile]["nameColour"]["r"]
				savedG = mrpSaved.Profiles[mrpSaved.SelectedProfile]["nameColour"]["g"]
				savedB = mrpSaved.Profiles[mrpSaved.SelectedProfile]["nameColour"]["b"]
			elseif(mrpSaved.Profiles["Default"]["nameColour"]) then
				savedR = mrpSaved.Profiles["Default"]["nameColour"]["r"]
				savedG = mrpSaved.Profiles["Default"]["nameColour"]["g"]
				savedB = mrpSaved.Profiles["Default"]["nameColour"]["b"]
			else
				savedR, savedG, savedB = 0.69, 0.29, 1 -- Magic values :3
			end
			mrp_ShowColourPicker(savedR, savedG, savedB, 1, mrp_NameColourChangedCallback)
		end )
		
		-- Eyes colour button
		mef.cepb = CreateFrame( "Button", "MyRolePlayCharacterFrame_EyeColourButton", mef, uipbt )
		local mrpColourButtonTexture = mef.cepb:CreateTexture("MyRolePlay_EyeColourButtonBackground", "BACKGROUND")
		mrpColourButtonTexture:SetAllPoints(mef.cepb)
		mrpColourButtonTexture:SetTexture(0.5,0.5,1)
		mef.cepb:SetPoint( "BOTTOMLEFT", mef.editbox, "BOTTOMLEFT", 20, -50 )
		mef.cepb:SetText( L["Change Eye Colour"] )
		mef.cepb:SetWidth( 150 )
		mef.cepb:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Add a colour to your eye field."], 1.0, 1.0, 1.0 )
		end )
		mef.cepb:SetScript( "OnLeave", GameTooltip_Hide )
		mef.cepb:SetScript("OnClick", function (self)
			local savedR, savedG, savedB
			if(mrpSaved.Profiles[mrpSaved.SelectedProfile]["eyeColour"]) then
				savedR = mrpSaved.Profiles[mrpSaved.SelectedProfile]["eyeColour"]["r"]
				savedG = mrpSaved.Profiles[mrpSaved.SelectedProfile]["eyeColour"]["g"]
				savedB = mrpSaved.Profiles[mrpSaved.SelectedProfile]["eyeColour"]["b"]
			elseif(mrpSaved.Profiles["Default"]["eyeColour"]) then
				savedR = mrpSaved.Profiles["Default"]["eyeColour"]["r"]
				savedG = mrpSaved.Profiles["Default"]["eyeColour"]["g"]
				savedB = mrpSaved.Profiles["Default"]["eyeColour"]["b"]
			else
				savedR, savedG, savedB = 0.9924675, 0.9924675, 0.9924675 -- Magic values :3
			end
			mrp_ShowColourPicker(savedR, savedG, savedB, 1, mrp_EyeColourChangedCallback)
		end )
		
		-- Restore name colour to default button
		mef.dfb = CreateFrame( "Button", "MyRolePlayCharacterFrame_RestoreDefaultColourButtonName", mef, uipbt )
		local mrpColourButtonTexture = mef.dfb:CreateTexture("MyRolePlay_ColourButtonBackground", "BACKGROUND")
		mrpColourButtonTexture:SetAllPoints(mef.dfb)
		mrpColourButtonTexture:SetTexture(0.5,0.5,1)
		mef.dfb:SetPoint( "RIGHT", mef.cpb, "RIGHT", 150, 0)
		mef.dfb:SetText( L["Restore Colour Default"] )
		mef.dfb:SetWidth( 150 )
		mef.dfb:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Restore your colour to the default profile's colour."], 1.0, 1.0, 1.0 )
		end )
		mef.dfb:SetScript( "OnLeave", GameTooltip_Hide )
		mef.dfb:SetScript("OnClick", function (self)
			mrpSaved.Profiles[mrpSaved.SelectedProfile]["nameColour"] = nil
			ColorPickerFrame:Hide()
			if(msp.my["NA"]) then
				msp.my["NA"] = string.gsub(msp.my["NA"], "|cff%x%x%x%x%x%x", "")
			end
			if(mrpSaved.Profiles[mrpSaved.SelectedProfile]["NA"]) then
				mrpSaved.Profiles[mrpSaved.SelectedProfile]["NA"] = string.gsub(mrpSaved.Profiles[mrpSaved.SelectedProfile]["NA"], "|cff%x%x%x%x%x%x", "")
			end
			if(MyRolePlayEditFrame) then
				local currentText = MyRolePlayEditFrame.editbox:GetText()
				currentText = string.gsub(currentText, "|cff%x%x%x%x%x%x", "")
				MyRolePlayEditFrame.editbox:SetText(currentText)
			end
			mrp:UpdateColour("NA")
			mrp:UpdateCharacterFrame()
		end )
		
		-- Restore eye colour to default button
		mef.dfbe = CreateFrame( "Button", "MyRolePlayCharacterFrame_RestoreDefaultColourButtonEyes", mef, uipbt )
		local mrpColourButtonTexture = mef.dfbe:CreateTexture("MyRolePlay_ColourButtonBackground", "BACKGROUND")
		mrpColourButtonTexture:SetAllPoints(mef.dfbe)
		mrpColourButtonTexture:SetTexture(0.5,0.5,1)
		mef.dfbe:SetPoint( "RIGHT", mef.cepb, "RIGHT", 150, 0)
		mef.dfbe:SetText( L["Restore Colour Default"] )
		mef.dfbe:SetWidth( 150 )
		mef.dfbe:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( L["Restore your colour to the default profile's colour."], 1.0, 1.0, 1.0 )
		end )
		mef.dfbe:SetScript( "OnLeave", GameTooltip_Hide )
		mef.dfbe:SetScript("OnClick", function (self)
			mrpSaved.Profiles[mrpSaved.SelectedProfile]["eyeColour"] = nil
			ColorPickerFrame:Hide()
			if(msp.my["AE"]) then
				msp.my["AE"] = string.gsub(msp.my["AE"], "|cff%x%x%x%x%x%x", "")
			end
			if(mrpSaved.Profiles[mrpSaved.SelectedProfile]["AE"]) then
				mrpSaved.Profiles[mrpSaved.SelectedProfile]["AE"] = string.gsub(mrpSaved.Profiles[mrpSaved.SelectedProfile]["AE"], "|cff%x%x%x%x%x%x", "")
			end
			if(MyRolePlayEditFrame) then
				local currentText = MyRolePlayEditFrame.editbox:GetText()
				currentText = string.gsub(currentText, "|cff%x%x%x%x%x%x", "")
				MyRolePlayEditFrame.editbox:SetText(currentText)
			end
			mrp:UpdateColour("AE")
			mrp:UpdateCharacterFrame()
		end )

		mef.ok = CreateFrame( "Button", "MyRolePlayEditFrameOK", mef, uipbt )
		mef.ok:SetPoint( "BOTTOMRIGHT", CharacterFrame, "BOTTOMRIGHT", -8, 7 )
		mef.ok:SetText( "Save" )
		mef.ok:SetWidth( 90 )
		mef.ok:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( "Save changes.", 1.0, 1.0, 1.0 )
		end )
		mef.ok:SetScript( "OnLeave", GameTooltip_Hide )
		mef.ok:SetScript("OnClick", function (self)
			local field = MyRolePlayEditFrame.field
			local newtext = MyRolePlayEditFrame.editbox:GetText()
			mrp:SaveField( field, newtext )
			MyRolePlayEditFrame:Hide()
		end )

		mef.cancel = CreateFrame( "Button", "MyRolePlayEditFrameCancel", mef, uipbt )
		mef.cancel:SetPoint( "RIGHT", mef.ok, "LEFT", -16, 0 )
		mef.cancel:SetText( "Cancel" )
		mef.cancel:SetWidth( 90 )
		mef.cancel:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( "Cancel changes and return to the way it was.", 1.0, 1.0, 1.0 )
		end )
		mef.cancel:SetScript( "OnLeave", GameTooltip_Hide )
		mef.cancel:SetScript("OnClick", function (self)
			if(MyRolePlayEditFrame.field == "NA") then                                                           -- We need to update msp.char now because it's not properly updating the colour in the tooltip when they hit cancel instead of save.
				local fullName = select(1, UnitFullName("player")) .. "-" .. select(2, UnitFullName("player")) -- Doing this way seems less risky than running mrp:SetCurrentProfile(), even if it'd be easier.
				if(msp.my["NA"]) then
					msp.char[fullName]["field"]["NA"] = msp.my["NA"]
				end
				if(msp.my["RC"]) then
					msp.char[fullName]["field"]["RC"] = msp.my["RC"]
				end
			end
			-- we, uh, don't need to do anything
			MyRolePlayEditFrame:Hide()
		end )

		mef.inherit = CreateFrame( "Button", "MyRolePlayEditFrameInherit", mef, uipbt )
		mef.inherit:SetPoint( "LEFT", mef )
		mef.inherit:SetPoint( "TOP", mef.ok )
		mef.inherit:SetText( "Inherit" )
		mef.inherit:SetWidth( 90 )
		mef.inherit:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( "Use the same contents as the Default profile has.", 1.0, 1.0, 1.0 )
		end )
		mef.inherit:SetScript( "OnLeave", GameTooltip_Hide )
		mef.inherit:SetScript("OnClick", function (self)
			local field = MyRolePlayEditFrame.field
			mrp:SaveField( field, nil )
			MyRolePlayEditFrame:Hide()
		end )
		mef.inherit:Hide()

		mef.inherited = mef:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
		mef.inherited:SetJustifyH( "LEFT" )
		mef.inherited:SetPoint( "LEFT", mef, "LEFT", 16, 0 )
		mef.inherited:SetPoint( "BOTTOMLEFT", mef, "BOTTOM", 0, -22 )
		mef.inherited:SetText( L["Inherited from Default."] )
		mef.inherited:Hide()
	end

	-- MyRolePlayComboEditFrame
	if not MyRolePlayComboEditFrame then
		local mef = CreateFrame( "Frame", "MyRolePlayComboEditFrame", MyRolePlayCharacterFrame )
		mef:Hide()
		mef:SetPoint( "TOPLEFT", MyRolePlayCharacterFrame, "TOPLEFT", 332, -61 )
		mef:SetPoint( "BOTTOM", CharacterFrameInset, "BOTTOM", 0, 30 )
		mef:SetPoint( "RIGHT", CharacterFrame, "RIGHT", -8, 0 )
		mef:EnableDrawLayer("ARTWORK")

		mef:SetScript("OnShow", function(self)
			CharacterFrame:SetWidth( 700 )
			CharacterFrame.Expanded = true
			UpdateUIPanelPositions(CharacterFrame)
			MyRolePlayComboEditFrame.editbox:SetFocus()
		end	)
		mef:SetScript("OnHide", function(self)
			CharacterFrame:SetWidth( PANEL_DEFAULT_WIDTH )
			CharacterFrame.Expanded = false
			UpdateUIPanelPositions(CharacterFrame)
		end	)
		mef.title = mef:CreateFontString( nil, "ARTWORK", "GameFontNormal" )
		mef.title:SetPoint( "TOP", mef, "TOP", 0, 27 )

		mef.desc = mef:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
		mef.desc:SetWordWrap(true)
		mef.desc:SetJustifyH( "LEFT" )
		mef.desc:SetJustifyV( "TOP" )
		mef.desc:SetPoint( "TOP", mef, "TOP", 0, -10 )

		mef:EnableDrawLayer("OVERLAY")

		mef.cb = CreateFrame( "Frame", "MyRolePlayComboEditFrameComboBox", mef, "UIDropDownMenuTemplate" )
		mef.cb:SetPoint( "CENTER", 0, 50 )
		UIDropDownMenu_SetWidth( mef.cb, 205 )

		mef.cb.dd = CreateFrame( "Button", "MyRolePlayComboEditFrameComboBoxDropDown", mef, "UIDropDownListTemplate" )

		MyRolePlayComboEditFrameComboBoxButton:SetScript( "OnClick", function( self )
			if DropDownList1:IsVisible() then
				DropDownList1:Hide()
			else
				EasyMenu( mrp.comboboxfields[ MyRolePlayComboEditFrame.field ], MyRolePlayComboEditFrame.cb.dd, MyRolePlayComboEditFrame.cb, 0, 5 )
				mrp.CFComboBoxUpdate( MyRolePlayComboEditFrame, true )
			end
		end )

		mef.editbox = CreateFrame( "EditBox", nil, mef )
		mef.editbox:SetBackdrop( {
				bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
				edgeFile = [[Interface\DialogFrame\UI-DialogBox-Border]],
				tile = true,
				tileSize = 32,
				edgeSize = 32,
				insets = { left = 11, right = 12, top = 12, bottom = 11	},
		} )
		mef.editbox:SetPoint( "BOTTOM", mef, "BOTTOM", 0, 30 )
		mef.editbox:SetHeight( 40 )
		mef.editbox:SetWidth( 205 )
		mef.editbox:SetMaxLetters( 35 )

		mef.editbox:SetTextInsets( 12, 12, 3, 3 )
		mef.editbox:EnableMouse(true)
		mef.editbox:SetAutoFocus(false)
		mef.editbox:SetMultiLine(false)
		mef.editbox:SetFontObject( "GameFontHighlight" )

		mef.editbox:SetScript( "OnEscapePressed", EditBox_ClearFocus )

		mef.ok = CreateFrame( "Button", "MyRolePlayComboEditFrameOK", mef, uipbt )
		mef.ok:SetPoint( "BOTTOMRIGHT", CharacterFrame, "BOTTOMRIGHT", -8, 7 )
		mef.ok:SetText( "Save" )
		mef.ok:SetWidth( 90 )
		mef.ok:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( "Save changes.", 1.0, 1.0, 1.0 )
		end )
		mef.ok:SetScript( "OnLeave", GameTooltip_Hide )
		mef.ok:SetScript("OnClick", function (self)
			if type( MyRolePlayComboEditFrame.value ) == "string" then
				MyRolePlayComboEditFrame.value = strtrim( MyRolePlayComboEditFrame.editbox:GetText() )
				if strtrim( MyRolePlayComboEditFrame.value ) == "" then 
					MyRolePlayComboEditFrame:Hide()
					return
				else
					local e = strlower( MyRolePlayComboEditFrame.value )
					for i = 0, 4 do 
						if e == strlower( L[ MyRolePlayComboEditFrame.field .. tostring( i ) ] ) then
							MyRolePlayComboEditFrame.value = i
							break
						end
					end 
				end
			end
			mrp:SaveField( MyRolePlayComboEditFrame.field, tostring( MyRolePlayComboEditFrame.value ) )
			MyRolePlayComboEditFrame:Hide()
		end )

		mef.cancel = CreateFrame( "Button", "MyRolePlayComboEditFrameCancel", mef, uipbt )
		mef.cancel:SetPoint( "RIGHT", mef.ok, "LEFT", -16, 0 )
		mef.cancel:SetText( "Cancel" )
		mef.cancel:SetWidth( 90 )
		mef.cancel:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( "Cancel changes and return to the way it was.", 1.0, 1.0, 1.0 )
		end )
		mef.cancel:SetScript( "OnLeave", GameTooltip_Hide )
		mef.cancel:SetScript("OnClick", function (self)
			-- we, uh, don't need to do anything
			MyRolePlayComboEditFrame:Hide()
		end )

		mef.inherit = CreateFrame( "Button", "MyRolePlayComboEditFrameInherit", mef, uipbt )
		mef.inherit:SetPoint( "LEFT", mef )
		mef.inherit:SetPoint( "TOP", mef.ok )
		mef.inherit:SetText( "Inherit" )
		mef.inherit:SetWidth( 90 )
		mef.inherit:SetScript( "OnEnter", function(self) 
			GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
			GameTooltip:SetText( "Use the same contents as the Default profile has.", 1.0, 1.0, 1.0 )
		end )
		mef.inherit:SetScript( "OnLeave", GameTooltip_Hide )
		mef.inherit:SetScript("OnClick", function (self)
			local field = MyRolePlayComboEditFrame.field
			mrp:SaveField( field, nil )
			MyRolePlayComboEditFrame:Hide()
		end )
		mef.inherit:Hide()

		mef.inherited = mef:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
		mef.inherited:SetJustifyH( "LEFT" )
		mef.inherited:SetPoint( "LEFT", mef, "LEFT", 16, 0 )
		mef.inherited:SetPoint( "BOTTOMLEFT", mef, "BOTTOM", 0, -22 )
		mef.inherited:SetText( L["Inherited from Default."] )
		mef.inherited:Hide()
	end
	-- Garbage-collect functions we only need once
	mrp.CreateEditFrames = mrp_dummyfunction
end

function mrp.CFComboBoxClick( self, iscustom )
	if iscustom then
		if MyRolePlayComboEditFrame.value == 0 then
			MyRolePlayComboEditFrame.value = ""
		else
			MyRolePlayComboEditFrame.value = emptynil( strtrim( MyRolePlayComboEditFrame.editbox:GetText() ) ) or mrp.Display[ MyRolePlayComboEditFrame.field ]( MyRolePlayComboEditFrame.value )
		end
	else
		MyRolePlayComboEditFrame.value = self.value
	end
	mrp.CFComboBoxUpdate( MyRolePlayComboEditFrame, true )
end

function mrp.CFComboBoxUpdate( mef, doset )
	if type( mef.value ) == "number" then
		if doset then UIDropDownMenu_SetSelectedID( MyRolePlayComboEditFrame.cb, MyRolePlayComboEditFrame.value + 1 ) end
		if mef.value == 0 then 
			mef.editbox:SetText( "" )
			mef.editbox:SetCursorPosition( 0 )
		else
			mef.editbox:SetText( mrp.comboboxfields[ mef.field ][ mef.value + 1 ].text )
			mef.editbox:SetCursorPosition( #mrp.comboboxfields[ mef.field ][ mef.value + 1 ].text )
		end
		mef.editbox:Hide()
	else
		if doset then UIDropDownMenu_SetSelectedID( MyRolePlayComboEditFrame.cb, 6 ) end
		mef.editbox:SetCursorPosition( 0 )
		mef.editbox:SetText( mef.value )
		mef.editbox:SetCursorPosition( #mef.value )
		mef.editbox:Show()
		mef.editbox:SetFocus()
	end
end

mrp.comboboxfields = {
	['FR'] = {
		{ text = L["FR0"], colorCode = "|cff808080", tooltipTitle = L["FR0t"], tooltipText = L["FR0d"], tooltipOnButton = 1, value = 0, func = mrp.CFComboBoxClick },
		{ text = L["FR1"], colorCode = "|cff66b380", tooltipTitle = L["FR1t"], tooltipText = L["FR1d"], tooltipOnButton = 1, value = 1, func = mrp.CFComboBoxClick },
		{ text = L["FR2"], colorCode = "|cff99b3cc", tooltipTitle = L["FR2t"], tooltipText = L["FR2d"], tooltipOnButton = 1, value = 2, func = mrp.CFComboBoxClick },
		{ text = L["FR3"], colorCode = "|cffe6ccb3", tooltipTitle = L["FR3t"], tooltipText = L["FR3d"], tooltipOnButton = 1, value = 3, func = mrp.CFComboBoxClick },
		{ text = L["FR4"], colorCode = "|cff99664d", tooltipTitle = L["FR4t"], tooltipText = L["FR4d"], tooltipOnButton = 1, value = 4, func = mrp.CFComboBoxClick },
		{ text = L["FRc"], tooltipTitle = L["FRct"], tooltipText = L["FRcd"], tooltipOnButton = 1, arg1 = true, func = mrp.CFComboBoxClick },
	},
	['FC'] = {
		{ text = L["FC0"], colorCode = "|cff808080", tooltipTitle = L["FC0t"], tooltipText = L["FC0d"], tooltipOnButton = 1, value = 0, func = mrp.CFComboBoxClick },
		{ text = L["FC1"], colorCode = "|cff99664d", tooltipTitle = L["FC1t"], tooltipText = L["FC1d"], tooltipOnButton = 1, value = 1, func = mrp.CFComboBoxClick },
		{ text = L["FC2"], colorCode = "|cff66b380", tooltipTitle = L["FC2t"], tooltipText = L["FC2d"], tooltipOnButton = 1, value = 2, func = mrp.CFComboBoxClick },
		{ text = L["FC3"], colorCode = "|cff99b3cc", tooltipTitle = L["FC3t"], tooltipText = L["FC3d"], tooltipOnButton = 1, value = 3, func = mrp.CFComboBoxClick },
		{ text = L["FC4"], colorCode = "|cffe6ccb3", tooltipTitle = L["FC4t"], tooltipText = L["FC4d"], tooltipOnButton = 1, value = 4, func = mrp.CFComboBoxClick },
		{ text = L["FCc"], tooltipTitle = L["FCct"], tooltipText = L["FCcd"], tooltipOnButton = 1, arg1 = true, func = mrp.CFComboBoxClick },
	},
}

-- When you click on a field: display and set up the appropriate edit frame
function mrp.CFEditField( field, fieldname, fielddesc )
		-- Advanced combination of both for glances, we want MyRolePlayGlanceEditFrame
	if field == 'PE' then
		MyRolePlayEditFrame:Hide()
		MyRolePlayMultiEditFrame:Hide()
		MyRolePlayComboEditFrame:Hide()

		local mef = MyRolePlayGlanceEditFrame
		mef.field = field
		
		local profile = mrpSaved.SelectedProfile
		
		if(type(mrpSaved.Profiles[profile]["glances"]) ~= "table") then -- If they don't have glances in their profile, make it.
			mrpSaved.Profiles[profile]["glances"] = {}
			for i = 1, 5, 1 do
				mrpSaved.Profiles[profile]["glances"][i] = {}
				mrpSaved.Profiles[profile]["glances"][i]["Title"] = _G["Glance" .. i .. "Title"]:GetText()
				mrpSaved.Profiles[profile]["glances"][i]["Description"] = _G["Glance" .. i .. "Descrip"].editbox:GetText()
				--mrpSaved.Profiles[profile]["glances"][i]["Icon"] = _G["Glance" .. i .. "IconTexture"]:GetTexture()
				mrpSaved.Profiles[profile]["glances"][i]["Icon"] = "Interface\\Icons\\INV_Misc_QuestionMark"
			end
		end
		
		if(profile and mrpSaved.Profiles[profile] and mrpSaved.Profiles[profile]["glances"]) then
			for i = 1, 5, 1 do
				if(mrpSaved.Profiles[profile]["glances"][i]) then
					_G["Glance" .. i .. "Title"]:SetText(mrpSaved.Profiles[profile]["glances"][i]["Title"])
					_G["Glance" .. i .. "Descrip"].editbox:SetText(mrpSaved.Profiles[profile]["glances"][i]["Description"])
					if(mrpSaved.Profiles[profile]["glances"][i]["Icon"] and mrpSaved.Profiles[profile]["glances"][i]["Icon"] ~= "") then
						previousGlanceIcons[i] = mrpSaved.Profiles[profile]["glances"][i]["Icon"]
						_G["Glance" .. i .. "IconTexture"]:SetTexture(mrpSaved.Profiles[profile]["glances"][i]["Icon"])
					else
						previousGlanceIcons[i] = "Interface\\Icons\\INV_Misc_QuestionMark"
						_G["Glance" .. i .. "IconTexture"]:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
					end
				end
			end
		end

		mef:Hide()
		mef:Show()
	elseif field == 'DE' or field == 'HI' or field == 'CO' or field == 'CU' then
		-- Multiple lines, we want MyRolePlayMultiEditFrame
		MyRolePlayGlanceEditFrame:Hide()
		MyRolePlayComboEditFrame:Hide()
		MyRolePlayEditFrame:Hide()
		local mef = MyRolePlayMultiEditFrame
		mef.field = field
		mef.fieldname = fieldname
		mef.fielddesc = fielddesc
		mef.title:SetText( fieldname )

		mef.sf.editbox:SetCursorPosition(0)
		ScrollFrame_OnScrollRangeChanged(MyRolePlayMultiEditFrameScrollFrame)

		local text = msp.my[field] or ""

		mef.sf.editbox:EnableKeyboard(true)

		if msp.my[field] == mrpSaved.Profiles.Default[field] and mrpSaved.SelectedProfile ~= "Default" then
			mef.sf.editbox:SetTextColor( 1.0, 1.0, 1.0, 0.4 )
			mef.inherit:Hide()
			mef.inherited:Show()
		else
			mef.sf.editbox:SetTextColor( 1.0, 1.0, 1.0, 1.0 )
			mef.inherited:Hide()
			if mrpSaved.SelectedProfile == "Default" then
				mef.inherit:Hide()
			else
				mef.inherit:Show()
			end
		end

		mef.sf.editbox:SetText( text )
		mef.sf.editbox:SetCursorPosition( #text )
		ScrollFrame_OnScrollRangeChanged(MyRolePlayMultiEditFrameScrollFrame)

		mef:Hide()
		mef:Show()
	elseif field == 'FR' or field == 'FC' then
		-- Combo box + custom text, we want MyRolePlayComboEditFrame
		MyRolePlayGlanceEditFrame:Hide()
		MyRolePlayMultiEditFrame:Hide()
		MyRolePlayEditFrame:Hide()
		local mef = MyRolePlayComboEditFrame
		mef.field = field
		mef.fieldname = fieldname
		mef.fielddesc = fielddesc
		mef.title:SetText( fieldname )
		mef.desc:SetText( fielddesc )

		mef.editbox:SetCursorPosition(0)

		local text = emptynil( msp.my[field] ) or "0"

		if text == "0" or text == "1" or text == "2" or text == "3" or text == "4" then
			mef.value = tonumber( text )
			MyRolePlayComboEditFrameComboBoxText:SetText( (mrp.comboboxfields[field][ tonumber( text ) + 1 ].colorCode or "") .. mrp.comboboxfields[field][ tonumber( text ) + 1 ].text )
			if text == "0" then
				mef.editbox:SetText( "" )
				mef.editbox:SetCursorPosition( 0 )
			else
				mef.editbox:SetText( mrp.comboboxfields[field][ tonumber( text ) + 1 ].text )
				mef.editbox:SetCursorPosition( #mrp.comboboxfields[field][ tonumber( text ) + 1 ].text )
			end
		else
			mef.value = text
			MyRolePlayComboEditFrameComboBoxText:SetText( mrp.comboboxfields[field][ 6 ].text )
			mef.editbox:SetText( text ) 
			mef.editbox:SetCursorPosition( #text )
		end

		mef.editbox:EnableKeyboard(true)

		if msp.my[field] == mrpSaved.Profiles.Default[field] and mrpSaved.SelectedProfile ~= "Default" then
			mef.editbox:SetTextColor( 1.0, 1.0, 1.0, 0.4 )
			mef.inherit:Hide()
			mef.inherited:Show()
		else
			mef.editbox:SetTextColor( 1.0, 1.0, 1.0, 1.0 )
			mef.inherited:Hide()
			if mrpSaved.SelectedProfile == "Default" then
				mef.inherit:Hide()
			else
				mef.inherit:Show()
			end
		end

		mrp.CFComboBoxUpdate( mef, false )

		mef:Hide()
		mef:Show()
	else
		-- Single line, we want MyRolePlayEditFrame
		MyRolePlayGlanceEditFrame:Hide()
		MyRolePlayMultiEditFrame:Hide()
		MyRolePlayComboEditFrame:Hide()

		local mef = MyRolePlayEditFrame
		mef.field = field
		mef.fieldname = fieldname
		mef.fielddesc = fielddesc
		mef.title:SetText( fieldname )
		mef.desc:SetText( fielddesc )

		mef.editbox:SetCursorPosition(0)
		
		local text = msp.my[field] or ""
		mef.editbox:EnableKeyboard(true)

		if msp.my[field] == mrpSaved.Profiles.Default[field] and mrpSaved.SelectedProfile ~= "Default" then
			mef.editbox:SetTextColor( 1.0, 1.0, 1.0, 0.4 )
			mef.inherit:Hide()
			mef.inherited:Show()
		else
			mef.editbox:SetTextColor( 1.0, 1.0, 1.0, 1.0 )
			mef.inherited:Hide()
			if mrpSaved.SelectedProfile == "Default" then
				mef.inherit:Hide()
			else
				mef.inherit:Show()
			end
		end

		mef.editbox:SetText( text )
		mef.editbox:SetCursorPosition( #text )

		mef:Hide()
		mef:Show()
	end
end
function mrp:HideEditFrames()
	MyRolePlayEditFrame:Hide()
	MyRolePlayMultiEditFrame:Hide()
	MyRolePlayComboEditFrame:Hide()
end

